@@ -85,7 +85,7 @@ extern int getnobody __((void));
 /* prversion.c */
 extern void prversion __((char *progname));
 
-#ifndef USE_GETPWNAM
+#ifdef USE_ZGETPWNAM
 /* pwdgrp.c */
 extern struct passwd	*zgetpwnam __((const char *name));
 extern struct passwd	*zgetpwuid __((const char *uid));
