@@ -138,14 +138,14 @@ struct received {
 	struct address	*r_id;
 	struct address	*r_for;
 	struct token	*r_convert;
-	long		r_time;
+	time_t		r_time;
 };
 
 union misc {					/* what a header can be */
 	struct address	*a;
 	struct token	*t;
 	struct received	*r;
-	long		d; /* time_t ?? */
+	time_t		d; 
 };
 
 #include "rfc822.entry"
@@ -217,7 +217,7 @@ struct envelope {
 	struct tm	e_localtime;	/* local time message arrived	*/
 	long		e_hdrOffset;	/* offset of start of headers	*/
 	long		e_msgOffset;	/* offset of message body	*/
-	long		e_nowtime;	/* time message processed	*/
+	time_t		e_nowtime;	/* time message processed	*/
 	int		e_resent;	/* flag indicating resent msg	*/
 	int		e_trusted;	/* is local agent trusted?	*/
 	struct conscell	*e_from_trusted;  /* given (channel,host,user)	*/
@@ -228,14 +228,14 @@ struct envelope {
 };
 
 #ifdef	RFC976
-#define HDR_SCANNER(x)	scan822(&(x), (long)strlen((char *)x), (u_char)'!', (u_char)'%', 0, (struct token **)0)
+#define HDR_SCANNER(x)	scan822(&(x), strlen((char *)x), (u_char)'!', (u_char)'%', 0, (struct token **)0)
 #else	/* !RFC976 */
-#define HDR_SCANNER(x)	scan822(&(x), (long)strlen((char *)x), (u_char)0, (u_char)0, 0, (struct token **)0)
+#define HDR_SCANNER(x)	scan822(&(x), strlen((char *)x), (u_char)0, (u_char)0, 0, (struct token **)0)
 #endif	/* RFC976 */
 
 /* #include "libz.h" */
 
-#ifndef	USE_GETPWNAM
+#ifdef	USE_ZGETPWNAM
 #define	getpwnam	zgetpwnam
 #define	getpwuid	zgetpwuid
 #define	getgrnam	zgetgrnam
