@@ -360,26 +360,58 @@ mail_close(fp)
 		++msgbase;
 
 	routerdir = ROUTERDIR;
+	nmessage  = NULL;
+	s         = NULL;
 	if (mail_priority) {
 	  /* We are asked to place the mail somewhere else */
 	  char *routerdirs = getzenv("ROUTERDIRS");
 	  if (routerdirs) {
-	    /* There are some defined!   A ":" separated list of strings */
 	    int i = mail_priority;
 	    char *rd = routerdirs;
-	    while ((s = strchr(rd,':')) && --i)
-	      rd = s+1;
+	    char *ord = routerdir;
+	    struct stat stb;
+	    nmessage = mail_alloc(strlen(postoffice)+
+				  strlen(routerdirs)+3+strlen(msgbase));
+	    /* There are some defined!   A ":" separated list of strings */
+
 	    /* mail_priority == 1 pics first, 2 pics second, ..
-	       if segments run out, last one is kept at  rs     */
-	    if (s) *s = 0;
-	    routerdir = rd;
+	       if segments run out, last one is kept at  rd     */
+
+	    while (i-- && (s = strchr(rd,':'))) {
+	      *s = 0;
+	      sprintf(nmessage, "%s/%s", postoffice, rd);
+	      *s = ':';
+	      if ((stat(nmessage,&stb) < 0) || !S_ISDIR(stb.st_mode)) {
+		rd = s+1;
+		continue;	/* Not ok -- not a dir, for example */
+	      }
+	      ord = rd;
+	      rd = s+1;
+	    }
+
+	    /* Here we are when there is only one entry in the routerdirs: */
+	    if (s == NULL && i > 0 && *rd != 0) {
+	      if (s) *s = 0;
+	      sprintf(nmessage, "%s/%s", postoffice, rd);
+	      if (s) *s = ':';
+	      /* Is it a valid directory ? */
+	      if ((stat(nmessage,&stb) == 0) && S_ISDIR(stb.st_mode))
+		ord = rd; /* IT IS ! */
+	    }
+	    routerdir = ord;
 	  }
 	}
 	/* Assert postoffice != NULL */
-	nmessage = mail_alloc(strlen(postoffice)+
-				    strlen(routerdir)+strlen(msgbase)+2+1);
-	sprintf(nmessage, "%s/%s/%s", postoffice, routerdir, msgbase);
-	if (s) *s = ':';
+	if (nmessage == NULL) {
+	  nmessage = mail_alloc(strlen(postoffice)+
+				strlen(routerdir)+strlen(msgbase)+2+1);
+	  sprintf(nmessage, "%s/%s/%s", postoffice, routerdir, msgbase);
+	} else {
+	  s = strchr(routerdir,':');
+	  if (s) *s = 0;
+	  sprintf(nmessage, "%s/%s/%s", postoffice, routerdir, msgbase);
+	  if (s) *s = ':';
+	}
 
 	/*
 	 * Unfortunately, rename() doesn't guarantee the same inode will
