@@ -152,7 +152,7 @@ regsub(prog, n)
 	if (prog == NULL || n < 0 || n > prog->re.re_nsub)
 		return (NULL);
 
-	return (prog->match[n]);
+	return (u_char*)(prog->match[n]);
 }
 
 STATIC regexp *
@@ -168,7 +168,7 @@ reg_comp (str)
 			progname, str);
 		return (NULL);
 	}
-	bzero(prog, sizeof(regexp));
+	bzero((void*)prog, sizeof(regexp));
 
 	prog->pattern = str;
 
