@@ -450,7 +450,8 @@ run_cadddr(avl, il)
 }
 
 static SIGNAL_TYPE
-sig_exit()
+sig_exit(sig)
+int sig;
 {
 	if (canexit) {
 #ifdef	MALLOC_TRACE
@@ -803,8 +804,10 @@ run_daemon(argc, argv)
 		dirp[i] = NULL; dirs[i] = NULL;
 	}
 	/* dirp[0] = opendir(".");	/* assert dirp != NULL ... */
+#if 0
 #ifdef BSD
 	dirp[0]->dd_size = 0;	/* stupid Berkeley bug workaround */
+#endif
 #endif
 	stickymem = MEM_MALLOC;
 	dirs[0] = strnsave("",1);
@@ -818,8 +821,10 @@ run_daemon(argc, argv)
 			/* strcat(pathbuf,"/"); */
 
 			if (stat(pathbuf,&stb) == 0  && S_ISDIR(stb.st_mode)) {
+#if 0
 #ifdef	BSD
 			  dirp[i]->dd_size = 0;
+#endif
 #endif
 			  dirs[i] = strsave(pathbuf);
 			  ++i;
@@ -1253,13 +1258,13 @@ run_listexpand(avl, il)
 	       cdr(il) != NULL && STRING(cdr(il))) {
 	  switch( il->string[1] ) {
 	    case 'c':
-	      comment = cdr(il)->string;
+	      comment = (char*)cdr(il)->string;
 	      break;
 	    case 'e':
-	      erroraddress = cdr(il)->string;
+	      erroraddress = (char*)cdr(il)->string;
 	      break;
 	    case 'E':
-	      errors_to = cdr(il)->string;
+	      errors_to = (char*)cdr(il)->string;
 	      break;
 	    default:
 	      errflag = 1;
@@ -1282,18 +1287,18 @@ run_listexpand(avl, il)
 	attributes = copycell(il);
 	car(attributes) = s_copy_tree(car(attributes));
 	cdr(attributes) = NULL;
-	localpart  = cdr(il)->string;
-	origaddr   = cddr(il)->string;
+	localpart  = (char*) cdr(il)->string;
+	origaddr   = (char*) cddr(il)->string;
 
 	/* We (memory-)leak this stuff for a moment.. */
 	e = (struct envelope *)tmalloc(sizeof (struct envelope));
 	e->e_nowtime = now;
-	e->e_file = car(avl)->string;
+	e->e_file = (char*) car(avl)->string;
 	/* we only need sensible `e' if its a time stamp header,
 	   which it ain't, so "listaddress" is just fine for e_file .. */
 
 	hs.h_descriptor = &aliashdr;
-	hs.h_pname = car(avl)->string;
+	hs.h_pname = (char*) car(avl)->string;
 
 	/* create h->h_lines */
 	/*
@@ -1384,7 +1389,7 @@ run_listexpand(avl, il)
 	if (c && (*s == ',' || *s == '\n'))
 		*s = '\0';
 	*pt = NULL;
-	s = buf; /* For debugger uses far below... */
+	s = (u_char*) buf; /* For debugger uses far below... */
 
 	hs.h_contents = hdr_scanparse(e, &hs, 1);
 	hs.h_stamp = hdr_type(&hs);
@@ -1432,8 +1437,8 @@ run_listexpand(avl, il)
 #define cdddr(x) cdr(cddr(x))
 
 		/* XX: We leak these at the moment! */
-		l       = newstring(strsave(buf));
-		cdr(l)  = conststring(origaddr);
+		l       = newstring((u_char*)strsave(buf));
+		cdr(l)  = conststring((u_char*)origaddr);
 		cddr(l) = car(attributes);
 		l = ncons(l);
 
@@ -1448,7 +1453,7 @@ run_listexpand(avl, il)
 		  s_value = NULL;
 		  l = conststring((u_char *)"hold");
 		  cdr(l)   = copycell(cdr(d));
-		  cddr(l)  = newstring(strsave(buf));
+		  cddr(l)  = newstring((u_char *)strsave(buf));
 		  cdddr(l) = car(attributes);
 		  l = ncons(l);
 		  l = ncons(l);
@@ -1527,7 +1532,7 @@ run_listexpand(avl, il)
 	if (al == NULL) { /* ERROR! NO ADDRESSES! */
 	  al = conststring((u_char *)"error");
 	  cdr(al)  = conststring((u_char *)"expansion");
-	  cddr(al) = newstring(strsave(localpart));
+	  cddr(al) = newstring((u_char*)strsave(localpart));
 	  al = ncons(al);
 	  al = ncons(al);
 	  errors_to = olderrors;
@@ -1882,7 +1887,7 @@ run_runas(argc, argv)
 	if (isdigit(*cp))	/* what if user id is "3com" ? */
 		uid *= atoi(cp);
 	else			/* look up login name and get uid */
-		uid = login_to_uid(cp);
+		uid = login_to_uid((u_char*)cp);
 
 	if (!initeduid)
 		myuid = geteuid();
@@ -1970,7 +1975,7 @@ run_login2uid(argc, argv)
 		fprintf(stderr, "Usage: %s login\n", argv[0]);
 		return 1;
 	}
-	printf("%d\n", login_to_uid(argv[1]));
+	printf("%d\n", login_to_uid((u_char*)argv[1]));
 	return 0;
 }
 
