@@ -86,7 +86,8 @@ reopen:
 		fm->membuf = NULL;
 #endif
 		if (spl == NULL)
-			sp_install(symid, (u_char *)fm, O_RDONLY, spt_files);
+			spl = sp_install(symid, (u_char *)fm,
+					 O_RDONLY, spt_files);
 		else
 			spl->data = (u_char *)fm;
 	}
@@ -141,6 +142,7 @@ reopen:
 		if (fm->offsets != NULL)
 		  free(fm->offsets);
 		free(fm);
+		spl->data = NULL;
 		++retry;
 		goto reopen;
 	}
@@ -169,8 +171,8 @@ close_seq(sip)
 	spl = sp_lookup(symid, spt_files);
 	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL)
 		return;	/* nothing to flush */
+	spl->data = NULL;	/* Delete the file-entry */
 	fclose(fm->fp);
-	sp_install(symid, (u_char *)NULL, 0, spt_files);
 #ifdef	USE_MMAP
 	if (fm->membuf != NULL)
 		munmap(fm->membuf,fm->size);
@@ -375,3 +377,133 @@ struct file_map *fm;
 	return buf;
 }
 #endif
+
+
+/* Indirect mappings -- like  aliases -- use this */
+
+struct conscell *
+readchunk(file, offset)
+	char *file;
+	long offset;
+{
+	FILE *fp;
+	register unsigned char *cp;
+	unsigned char *as;
+	struct conscell *tmp, *l;
+	struct spblk *spl;
+	int retry, symid, flag, len;
+	char buf[BUFSIZ];
+	struct file_map *fm;
+#ifdef USE_MMAP
+	struct stat fst;
+#endif
+
+	if (file == NULL || offset < 0)
+		return NULL;
+	
+	retry = 0;
+	symid = symbol((u_char *)file);
+	spl = sp_lookup(symid, spt_files);
+	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
+reopen:
+		fp = fopen(file, "r");
+		if (fp == NULL) {
+			++deferit;
+			v_set(DEFER, DEFER_IO_ERROR);
+			fprintf(stderr, "search_seq: cannot open %s!\n", file);
+			return NULL;
+		}
+		fm = (struct file_map*)emalloc(sizeof(struct file_map));
+		/* No MMAP() of this data! */
+		fm->fp = fp;
+		fm->size   = 0;
+		fm->mtime  = 0;
+		fm->lines = 0;
+		fm->offsets = NULL;
+		fm->membuf = NULL;
+		if (spl == NULL)
+			spl = sp_install(symid, (u_char *)fm,
+					 O_RDONLY, spt_files);
+		else
+			spl->data = (u_char *)fm;
+	}
+	fp = fm->fp;
+
+	if (fseek(fp, (long)offset, 0) != 0) {
+		++deferit;
+		v_set(DEFER, DEFER_IO_ERROR);
+		fprintf(stderr,
+			"indirect postprocessor: bad seek (%ld) on '%s'!\n",
+			(long)offset, file);
+		return NULL;
+	}
+
+	len = 0;
+	flag = 0;
+	buf[sizeof(buf) - 1] = '\0';
+	while (fgets(buf, sizeof(buf)-1, fp) != NULL) {
+		/* tab and space are valid continuation characters */
+		if (flag && buf[0] != '\t' && buf[0] != ' ')
+			break;
+		if (buf[sizeof buf - 1] == '\0')
+			len += strlen(buf)-1;
+		else if (buf[sizeof buf - 1] == '\n')
+			len += sizeof buf - 1;
+		else
+			len += sizeof buf;
+		flag = 1;
+	}
+
+	if (fseek(fp, (long)offset, 0) != 0) {
+		++deferit;
+		v_set(DEFER, DEFER_IO_ERROR);
+		fprintf(stderr,
+			"indirect postprocessor: bad seek (%ld) on '%s'!\n",
+			(long)offset, file);
+		return NULL;
+	}
+
+	cp = as = (u_char *)tmalloc(len+1);
+	l = NULL;
+	flag = 0;
+	buf[sizeof buf - 1] = '\0';
+	while (fgets(buf, sizeof buf, fp) != NULL) {
+		/* printaliases (actually hdr_print()) prints lines < 80 char */
+
+		if (buf[0] == '\t')
+			buf[0] = ' ';
+		else if (flag && buf[0] != ' ')
+			break;
+
+		if (buf[sizeof buf - 1] == '\0') {
+			len = strlen(buf)-1;
+			strncpy((char *)cp, buf, len);
+			cp += len;
+		} else if (buf[sizeof buf - 1] == '\n') {
+			strncpy((char *)cp, buf, sizeof buf - 1);
+			cp += sizeof buf - 1;
+		} else {
+			strncpy((char *)cp, buf, sizeof buf);
+			cp += sizeof buf;
+		}
+		flag = 1;
+	}
+	if (cp > as) {
+		*cp = 0;
+		l = newstring(as);
+	}
+	if (!retry && ferror(fp)) {
+		if (l != NULL) {
+			if (stickymem == MEM_MALLOC)
+				s_free_tree(l);
+			l = NULL;
+		}
+		fclose(fp);
+		free(fm);
+		spl->data = NULL;
+		++retry;
+		goto reopen;
+	}
+
+	return l;
+}
