@@ -114,7 +114,7 @@ extern char	*gs_name;
 extern void	optsave __((int type, struct envelope *e));
 
 /* File: rfc822.c */
-extern u_char	*errors_to;
+extern char	*errors_to;
 
 extern int	run_rfc822 __((int argc, char **argv));
 extern int	makeLetter __((struct envelope *e, int octothrp));
@@ -143,8 +143,8 @@ extern struct headerinfo *senderDesc __((void));
 extern void	set_pname __((struct envelope *e, struct header *h, char *s));
 extern struct header	*copySender __((struct envelope *e));
 extern struct header	*copyRecipient __((struct header *h));
-extern struct header	*mkMessageId __((struct envelope *e, long unixtime));
-extern struct header	*mkDate __((int isresent, long unixtime));
+extern struct header	*mkMessageId __((struct envelope *e, time_t unixtime));
+extern struct header	*mkDate __((int isresent, time_t unixtime));
 extern void	hdr_print __((struct header *h, FILE *fp));
 extern int	hdr_nilp __((struct header *h));
 extern void	pureAddress __((FILE *fp, struct addr *pp));
@@ -174,7 +174,7 @@ extern int   router_id;
 extern int   savefile;
 
 extern int	main __((int argc, char *argv[]));
-extern short	login_to_uid __((u_char *name));
+extern int	login_to_uid __((u_char *name));
 extern u_char	*uidpwnam __((int uid));
 extern int	isgoodguy __((int uid));
 extern void	logmessage __((struct envelope *e));
