@@ -270,7 +270,7 @@ copyRecipient(h)
 struct header *
 mkMessageId(e, unixtime)
 	struct envelope *e;
-	long unixtime;
+	time_t unixtime;
 {
 	register struct header *h;
 	static struct headerinfo *msgidDesc;
@@ -323,7 +323,7 @@ mkMessageId(e, unixtime)
 struct header *
 mkDate(isresent, unixtime)
 	int isresent;
-	long unixtime;
+	time_t unixtime;
 {
 	register struct header	*h;
 	u_char *cp, *s;
@@ -366,7 +366,7 @@ hdr_print(h, fp)
 	struct token *t;
 	struct address *ap;
 	struct addr *pp;
-	char *cp, *s;
+	/* char *cp, *s; */
 
 	if (h == NULL)
 		return;
@@ -426,11 +426,14 @@ hdr_print(h, fp)
 		}
 		putc(';', fp);
 		putc(' ', fp);
-		cp = rfc822date(&(h->h_contents.r->r_time));
-		s = cp + strlen(cp) -1;
-		if (*cp != 0 && *s == '\n') *s = 0;
-		fputs(cp, fp);
-		putc('\n', fp);
+		{
+		  char *cp, *s;
+		  cp = (char*)rfc822date(&(h->h_contents.r->r_time));
+		  s = cp + strlen(cp) -1;
+		  if (*cp != 0 && *s == '\n') *s = 0;
+		  fputs(cp, fp);
+		  putc('\n', fp);
+		}
 		break;
 	case nilHeaderSemantics:
 		for (t = h->h_lines; t != NULL; t = t->t_next) {
@@ -491,6 +494,7 @@ hdr_print(h, fp)
 	case DateTime:
 		/* Use this ONLY if no original lines are available! */
 		if (h->h_contents.d > 0L && h->h_lines == NULL) {
+			char *cp, *s;
 			cp = (char *)rfc822date(&(h->h_contents.d));
 			s = cp + strlen(cp) -1;
 			if (*cp != 0 && *s == '\n') *s = 0;
@@ -499,6 +503,7 @@ hdr_print(h, fp)
 			break;
 		} /* else we have the original line ... */
 		for (t = h->h_lines; t != NULL; t = t->t_next) {
+			char *cp, *s;
 			cp = (char *)t->t_pname;
 			/* Skip over the leading white-space */
 			while (*cp == ' '||*cp == '\t') ++cp;
