@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 #include <fcntl.h>
 #include "mail.h"
 #include "scheduler.h"
@@ -243,7 +244,7 @@ reporterrs(cfpi)
 	  /* Scan to the start of the message text */
 	  while (isascii(*cp) && isspace(*cp))
 	    ++cp;
-	  fprintf(errfp, "<%s>: ", cfp->contents + byteidx + 2);
+	  fprintf(errfp, "<%s>: ", cfp->contents + byteidx + 2 + _CFTAG_RCPTPIDSIZE);
 	  if (strchr(cp, '\r')) {
 	    fprintf(errfp, "...\\\n\t");
 	    for (s = cp; *s != '\0'; ++s) {
