@@ -7,16 +7,17 @@
  * ZMailer transport scheduler.
  */
 
+#include <stdio.h>
 #include <sys/param.h>
 #include "hostenv.h"
-#include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
 #include "scheduler.h"
 #include <sys/stat.h>
 #include <syslog.h>
 #include <fcntl.h>
-#include <mail.h>
+#include "mail.h"
+#include <string.h>
 #include NDIR_H
 #include "zmsignal.h"
 
@@ -66,7 +67,8 @@ time_t	next_delaysweep;
 #include "memtypes.h"
 extern memtypes stickymem;
 
-extern SIGNAL_TYPE sig_exit(), sig_alarm(), sig_iot(), sig_readcf();
+extern SIGNAL_TYPE sig_exit(), sig_alarm(), sig_iot(),
+		   sig_readcf(), sig_chld();
 extern char *Version;
 extern char *optarg;
 extern char *progname;
@@ -75,8 +77,8 @@ extern char *qcf_suffix;
 extern char *qlogdir;
 extern char *qoutputfile;
 extern char *strerror __((int errno));
-/* extern char *strchr __((char *s, char c));
-   extern char *strrchr __((char *s, char c)); */
+extern char *strchr  /* __((char *s, char c)); */ ();
+extern char *strrchr /* __((char *s, char c)); */ ();
 extern char *vtx_to_dir __((struct vertex *vp, char *buf));
 extern int cistrcmp __((char *s1, char *s2));
 extern int efstat __((int fd, struct stat *stbuf));
@@ -87,7 +89,7 @@ extern int errno;
 extern int estat __((char *filename, struct stat *stbuf));
 extern int eunlink __((char *filename));
 extern int lockaddr __((int fd, long offset, char tag1, char tag2));
-extern int loginit __((void));
+extern int loginit __((int sig));
 extern int match __((char *pattern, char *string));
 extern int numkids;
 extern int optind;
@@ -119,6 +121,7 @@ extern void vtxdo __((struct vertex *vp, struct config_entry *ce));
 extern void vtxprint __((struct vertex *vp));
 extern void defaultconfigentry __((struct config_entry *ce, struct config_entry *));
 
+int global_maxkids = 1000;
 
 int
 main(argc, argv)
@@ -128,7 +131,8 @@ main(argc, argv)
 	struct ctlfile *cfp;
 	char *config, *cp;
 	int i, daemonflg, c, errflg, version, fd;
-	long now, offout, offerr;
+	long offout, offerr;
+	time_t now;
 	struct config_entry *tp;
 
 	/* setlinebuf(stderr);  -- no need for this ? */
@@ -142,6 +146,11 @@ main(argc, argv)
 
 	resources_maximize_nofiles();
 
+	/* The theory is, that scheduler needs circa 10 fd's for its own uses,
+	   and it will use all others on child-process return channels. */
+
+	global_maxkids = resources_query_nofiles()-10;
+
 	postoffice = rendezvous = log = config = NULL;
 	daemonflg = 1;
 	dlyverbose = 0;
@@ -208,15 +217,23 @@ main(argc, argv)
 	}
 	if (log != NULL) {
 		/* loginit is a signal handler, so can't pass log */
-		if (loginit() < 0)	/* do setlinebuf() there */
+		if (loginit(SIGHUP) < 0)	/* do setlinebuf() there */
 			die(1, "log initialization failure");
 		/* close and reopen log files */
-		SIGNAL_HANDLE(SIGHUP, (SIGNAL_TYPE (*)())loginit);
+		SIGNAL_HANDLE(SIGHUP, loginit);
 	} else {
 		SIGNAL_IGNORE(SIGHUP); /* no surprises please */
 		setvbuf(stdout, (char *)NULL, _IOLBF, 0);
 		setbuf(stderr,NULL); /* [mea] No buffering on stdout.. */
 	}
+#ifdef USE_SIGREAPER
+#ifdef SIGCLD
+	SIGNAL_HANDLE(SIGCLD,  sig_chld);
+#else
+	SIGNAL_HANDLE(SIGCHLD, sig_chld);
+#endif
+#endif
+
 #ifdef	SIGUSR1
 	SIGNAL_HANDLE(SIGUSR1, sig_readcf);
 #endif	/* SIGUSR1 */
@@ -259,7 +276,7 @@ main(argc, argv)
 			/* NOTREACHED */
 		}
 		detach();		/* leave worldy matters behind */
-		now = time((long *)0);
+		now = time(NULL);
 		printf("%s: scheduler daemon (%s)\n\tstarted at %s\n",
 		       progname, Version, (char *)rfc822date(&now));
 		killprevious(SIGTERM, pidfile);
@@ -378,7 +395,8 @@ main(argc, argv)
 }
 
 SIGNAL_TYPE
-sig_exit()
+sig_exit(sig)
+int sig;
 {
 	
 	if (querysocket >= 0) {		/* give up mailq socket asap */
@@ -391,7 +409,8 @@ sig_exit()
 }
 
 SIGNAL_TYPE
-sig_alarm()
+sig_alarm(sig)
+int sig;
 {
 	gotalarm = 1;
 	SIGNAL_HANDLE(SIGALRM, sig_alarm);
@@ -399,7 +418,8 @@ sig_alarm()
 
 #ifdef SIGUSR2
 SIGNAL_TYPE
-sig_iot()
+sig_iot(sig)
+int sig;
 {
 	dumpq = 1;
 	SIGNAL_HANDLE(SIGUSR2, sig_iot);
@@ -408,7 +428,8 @@ sig_iot()
 
 #ifdef SIGUSR1
 SIGNAL_TYPE
-sig_readcf()
+sig_readcf(sig)
+int sig;
 {
 	rereadcf = 1;
 	SIGNAL_HANDLE(SIGUSR1, sig_readcf);
@@ -1150,11 +1171,11 @@ vtxdo(vp, cehead)
 	if (vp->ce.interval == -1)
 	  vp->ce.interval = 3600;
 	if (vp->ce.maxkids == -1)
-	  vp->ce.maxkids = 1000;
+	  vp->ce.maxkids = global_maxkids;
 	if (vp->ce.maxkidChannel == -1)
-	  vp->ce.maxkidChannel = 1000;
+	  vp->ce.maxkidChannel = global_maxkids;
 	if (vp->ce.maxkidHost == -1)
-	  vp->ce.maxkidHost = 1000;
+	  vp->ce.maxkidHost = global_maxkids;
 	if (vp->ce.byhost == -1)
 	  vp->ce.byhost = 0;
 	if (vp->ce.bychannel == -1)
