@@ -3,6 +3,9 @@
  *	This will be free software, but only when it is finished.
  */
 
+#define USE_SIGREAPER /* Use SIGCLD-driven reaper.. */
+
+
 #include "hostenv.h"
 #include "malloc.h"
 #include "splay.h"
@@ -85,6 +88,7 @@ struct web {
 
 struct procinfo {
 	int	pid;
+	int	reaped;
 	struct web *ch;
 	struct web *ho;
 	struct vertex *vertices;
@@ -117,4 +121,3 @@ struct vertex {
 };
 
 extern char		*strcpy(), *strncpy(), *strcat();
-
