@@ -3,6 +3,7 @@
  *	This will be free software, but only when it is finished.
  */
 
+
 #include <stdio.h>
 #include <sys/param.h>
 #include "hostenv.h"
@@ -12,14 +13,9 @@
 #include <errno.h>
 #include <sys/file.h>
 #include NDIR_H
-/*
- * FD_SETSIZE should be set in either <sys/param.h> or
- * <sys/types.h> if we have USE_SELECT
- */
-#ifndef FD_SETSIZE
-#define	FD_SETSIZE	32
-#endif	/* !FD_SETSIZE */
+#include "zmsignal.h"
 
+int scheduler_nofiles = 32; /* Will be filled below */
 
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
@@ -30,7 +26,6 @@
 
 struct procinfo *cpids = NULL;
 
-
 #define	MAXFILESPERTRANSPORT	1000
 
 int	numkids = 0;
@@ -320,21 +315,26 @@ stashprocess(pid, fd, chwp, howp, vhead)
 		  pid, fd, chwp ? chwp->name : "nil",
 		  howp ? howp->name : "nil");
 	if (cpids == NULL) {
-	  i = FD_SETSIZE;
+	  scheduler_nofiles = i = resources_query_nofiles();
 	  cpids = (struct procinfo *)
 	    emalloc((unsigned)(i * sizeof (struct procinfo)));
 	  while (--i >= 0)
 	    cpids[i].pid = 0;
 	}
-	cpids[fd].pid = pid;
-	cpids[fd].ch = chwp;
-	cpids[fd].ho = howp;
+	cpids[fd].pid    = pid;
+	cpids[fd].reaped = 0;
+	cpids[fd].ch     = chwp;
+	cpids[fd].ho     = howp;
 	cpids[fd].vertices = vhead;
 	vhead->proc = &cpids[fd];
 	cpids[fd].carryover = NULL;
-#ifdef	USE_BSDNODELAY
+#ifdef O_NONBLOCK /* POSIXy thing */
 	/* set up non-blocking I/O */
 	i = fcntl(fd, F_GETFL, 0);
+	i |= O_NONBLOCK;
+	i = fcntl(fd, F_SETFL, i);
+#else
+#ifdef	USE_BSDNODELAY
 #ifdef	FNONBLOCK
 	i |= FNONBLOCK;
 #else
@@ -342,6 +342,7 @@ stashprocess(pid, fd, chwp, howp, vhead)
 #endif
 	i = fcntl(fd, F_SETFL, i);
 #endif	/* USE_BSDNODELAY */
+#endif	/* O_NONBLOCK  */
 }
 
 void
@@ -351,6 +352,7 @@ reclaim(fd)
 	struct vertex *vp, *nvp;
 
 	cpids[fd].pid = 0;
+	cpids[fd].reaped = 0;
 	if (cpids[fd].carryover != NULL) {
 	  fprintf(stderr, "%s: HELP! Lost %d bytes: '%s'\n",
 		  progname, strlen(cpids[fd].carryover),
@@ -393,6 +395,7 @@ void
 waitandclose(fd)
 	int	fd;
 {
+#ifndef USE_SIGREAPER
 	int	n, pid;
 
 #if 0
@@ -424,7 +427,7 @@ waitandclose(fd)
 		reclaim(fd);
 		break;
 	      } else {
-		for (n = 0; n < FD_SETSIZE; ++n) {
+		for (n =; n < scheduler_nofiles; ++n) {
 		  if (cpids[n].pid == pid) {
 		    cpids[n].pid = -pid;
 		    ++numkids;
@@ -438,6 +441,14 @@ waitandclose(fd)
 	  reclaim(fd);
 	}
 #endif
+#else	/* USE_SIGREAPER */
+	/* This is called when
+	   - fd return 0 (EOF)
+	   - fd returns -1, and errno != {EAGAIN|EWOULDBLOCK|EINTR}
+	 */
+	--numkids;
+	reclaim(fd);
+#endif
 }
 
 #ifdef	USE_SELECT
@@ -624,9 +635,9 @@ queryipcinit()
 	  return;
 	}
 	qipcretry = now + 5;
-	sad.sin_port = serv->s_port;
-	sad.sin_family = AF_INET;
-	sad.sin_addr.s_addr = INADDR_ANY;
+	sad.sin_port        = serv->s_port;
+	sad.sin_family      = AF_INET;
+	sad.sin_addr.s_addr = htonl(INADDR_ANY);
 	if ((querysocket = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	  perror("socket");
 	  return;
@@ -708,6 +719,7 @@ readfrom(fd)
 	  dontbreak = 1;
 	}
 	/* Note that if we get an alarm() call, the read will return -1, TG */
+	errno = 0;
 	while ((n = read(fd, dontbreak ? cp : buf,
 			 bufsize - (dontbreak ? (cp - buf) : 0))) > 0) {
 	  if (verbose)
@@ -752,8 +764,9 @@ readfrom(fd)
 	if (verbose) {
 	  fprintf(stderr, "read from %d returns %d, errno=%d\n", fd, n, errno);
 	}
-	if (n == 0 || (n < 0 && errno != EINTR && errno != EWOULDBLOCK
-		       && errno != EAGAIN)) {
+	if (n == 0 || (n < 0 && errno != EWOULDBLOCK && errno != EAGAIN &&
+		       errno != EINTR)) {
+	  /*printf("about to call waitandclose(), n=%d, errno=%d\n",n,errno);*/
 	  waitandclose(fd);
 	}
 	/* fprintf(stderr, "n = %d, errno = %d\n", n, errno); */
@@ -804,24 +817,25 @@ int
 system(name)
 	char *name;
 {
-        char *sh;
-        int st, r;
-        int pid;
-        int i;
+	char *sh;
+	int st, r;
+	int pid;
+	int i;
 
-        pid = fork();
-        switch(pid) {
-        case -1:
-                return -1;
-        case 0:
-                sh = getenv("SHELL");
-                if (sh == NULL) {
+	pid = fork();
+	switch(pid) {
+	case -1:
+		return -1;
+	case 0:
+		sh = getenv("SHELL");
+		if (sh == NULL) {
 		  sh = "/bin/sh";
-                }
-                execl(sh, sh, "-c", name, NULL);
-                _exit(1);
-        default:
-                while(1) {
+		}
+		execl(sh, sh, "-c", name, NULL);
+		_exit(1);
+	default:
+#ifndef USE_SIGREAPER
+		while(1) {
 		  r = wait(&st);
 		  if (r == -1) {
 		    if (errno != EINTR) {
@@ -843,9 +857,71 @@ system(name)
 		  if ((st & 0x00ff) == 0) {
 		    return st >> 8;
 		  }
-
 		  return 1;
 		}
+#endif
+		break;
+	   }
 }
 
 #endif
+
+
+#ifdef USE_SIGREAPER
+/*
+ *	Catch each child-process death, and reap them..
+ */
+SIGNAL_TYPE sig_chld(signum)
+int signum;
+{
+	int pid;
+	int ok = 0;
+	int i;
+#ifdef USE_UNIONWAIT
+	union wait statloc;
+#else
+	int statloc;
+#endif
+	pid = wait(&statloc);
+
+#ifdef	WIFEXITED
+	if (WIFEXITED(statloc)) ok = 1;
+#else
+      ERROR:ERROR:ERROR
+#endif
+#ifdef WIFSIGNALED
+	if (WIFSIGNALED(statloc)) ok = 1;
+#else
+      ERROR:ERROR:ERROR
+#endif
+
+	if (verbose)
+	  fprintf(stderr,"sig_chld() pid=%d, ok=%d\n",pid,ok);
+
+	if (ok && cpids != NULL) {
+	  /* Only EXIT and SIGxx DEATHS accepted */
+
+	  for (i = scheduler_nofiles-1; i >= 0; --i) {
+	    if (cpids[i].pid == pid) {
+	      cpids[i].pid = -pid;	/* Mark it as reaped.. */
+	      ok = 0;
+	      break;
+	    }
+	    if (cpids[i].pid == -pid) {
+printf(" .. already reaped ??\n");
+	      cpids[i].pid = -pid;	/* Mark it as reaped.. */
+	      ok = 0;
+	      break;
+	    }
+	  }
+	  if (ok)
+	    --numkids; /* A non-transporter-child (mailq() ?) */
+	}
+	/* re-instantiate the signal handler.. */
+#ifdef SIGCLD
+	SIGNAL_HANDLE(SIGCLD,  sig_chld);
+#else
+	SIGNAL_HANDLE(SIGCHLD, sig_chld);
+#endif
+}
+#endif /* USE_SIGREAPER */
