@@ -65,6 +65,10 @@ update(diagnostic)
 	int	inum, offset;
 	struct diagcodes *dcp;
 
+	if (*diagnostic == '#' /* DEBUG comment.. */) {
+	  fprintf(stderr, "DBGdiag: %s\n", diagnostic);
+	  return;
+	}
 	inum = atoi(diagnostic);
 	if ((cp = strchr(diagnostic, '/')) == NULL) {
 	  fprintf(stderr, "Misformed diagnostic: %s\n", diagnostic);
@@ -543,7 +547,7 @@ u_ok(inum, offset, notary, message)
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, 0, 2);
 	  fprintf(vp->cfp->vfp, "%s: ok %s\n",
-		  vp->cfp->contents + offset + 2,
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message == NULL ? "(sent)" : message);
 	}
 
@@ -590,7 +594,7 @@ u_deferred(inum, offset, notary, message)
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, 0, 2);
 	  fprintf(vp->cfp->vfp, "%s: deferred %s\n",
-		  vp->cfp->contents + offset + 2,
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message == NULL ? "(unknown)" : message);
 	}
 	/*
@@ -632,7 +636,7 @@ u_error(inum, offset, notary, message)
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, 0, 2);
 	  fprintf(vp->cfp->vfp, "%s: error %s\n",
-		  vp->cfp->contents + offset + 2,
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message);
 	}
 	vtxupdate(vp, index);
@@ -690,8 +694,8 @@ u_retryat(inum, offset, notary, message)
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, 0, 2);
 	  fprintf(vp->cfp->vfp, "%s: retryat %d %s\n",
-		  vp->cfp->contents + offset + 2, retrytime,
-		  message == NULL ? "(unknown)" : message);
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
+		  retrytime, message == NULL ? "(unknown)" : message);
 	}
 	/*
 	 * Even though we may get several of these per web entry,
