@@ -47,28 +47,19 @@ char *okstr;
 	}
 }
 
-/*VARARGS*/
 void
-notaryreport(va_alist) /* ( notaryargs[1,2,3] ) */
-	va_dcl
-{
-	va_list ap;
-	char *fmt = "%s\001%s\001%s";
+notaryreport(arg1,arg2,arg3)
 	char *arg1, *arg2, *arg3;
+{
+	char *fmt = "%s\001%s\001%s";
 	int len;
 	static char *A1 = NULL, *A2 = NULL, *A3 = NULL;
 
-	va_start(ap);
-	arg1 = va_arg(ap, char *);
-	arg2 = va_arg(ap, char *);
-	arg3 = va_arg(ap, char *);
-	va_end(ap);
-
 	if (arg1) { if (A1) free(A1); A1 = strdup(arg1); }
 	if (arg2) { if (A2) free(A2); A2 = strdup(arg2); }
 	if (arg3) { if (A3) free(A3); A3 = strdup(arg3); }
 
-	len = 22; /* "(null)\001(null)\001(null)" */
+	len = 4; /* "\001\001" */
 	if (A1) len += strlen(A1);
 	if (A2) len += strlen(A2);
 	if (A3) len += strlen(A3);
