@@ -566,9 +566,9 @@ deliver(dp, rp, timestring)
 	  if (uid == nobody) {
 	    if (propably_x400(rp->addr->user)) {
 	      notaryreport(rp->addr->user,"failed",
-			   "550 (this feels like X.400 address -- no support for them)");
+			   "550 (this feels like a misplaced X.400 address -- no support for them)");
 	      DIAGNOSTIC(rp, EX_UNAVAILABLE,
-			 "This appears to be a X.400 address, no support for them", 0);
+			 "This appears to be a misplaced X.400 address, no support for them", 0);
 	    } else {
 	      notaryreport(rp->addr->user,"failed",
 			   "550 (mail to file disallowed w/o priviledges)");
@@ -607,10 +607,18 @@ deliver(dp, rp, timestring)
 	    }
 	    if ((pw = getpwnam(rp->addr->user)) == NULL) {
 	      if (at) *at = '@';
-	      notaryreport(rp->addr->user,"failed","550 (User does not exist)");
-	      if (at) *at = 0;
-	      DIAGNOSTIC(rp, EX_NOUSER,
-			 "user \"%s\" doesn't exist", rp->addr->user);
+	      if (propably_x400(rp->addr->user)) {
+		notaryreport(rp->addr->user,"failed",
+			     "550 (this feels like a misplaced X.400 address -- no support for them)");
+		DIAGNOSTIC(rp, EX_UNAVAILABLE,
+			   "This appears to be a misplaced X.400 address, no support for them", 0);
+	      } else {
+		notaryreport(rp->addr->user,"failed","550 (User does not exist)");
+		if (at) *at = 0;
+		DIAGNOSTIC(rp, EX_NOUSER,
+			   "user \"%s\" doesn't exist", rp->addr->user);
+	      }
+	      if (at) *at = '@';
 	      return;
 	    }
 	  }
