@@ -6,7 +6,23 @@
 
 #define	RFC974		/* If BIND, check that TCP SMTP service is enabled */
 
-#define	TIMEOUT		(10*60)		/* timeout in seconds, per exchange */
+#define	TIMEOUT		(5*60)		/* timeout in seconds, per exchange */
+#define ALARM_BLOCKSIZE 1024
+#define ALARM_DOTTOOK   (60*60)
+			/* RFC 1123 recommends:
+			     - Initial 220: 5 minutes
+			     - MAIL, RCPT : 5 minutes
+			     - DATA initialization (until "354.."): 2 minutes
+			     - While writing data, a block
+			       at the time: 3 minutes  (How large a block ?)
+			     - From "." to "250 OK": 10 minutes
+
+			   I think we simplify:  5 minutes each, except "."
+			   to "250 ok" which is 60 (!) minutes. (sendmail's
+			   default value.)
+			   Block-size is 1kB.   4-Feb-95: [mea@utu.fi]
+
+			 */
 
 #include <stdio.h>
 #include "hostenv.h"
@@ -22,6 +38,9 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include NDIR_H
+#ifdef USE_ALLOCA
+# include <alloca.h>
+#endif
 
 #include <netdb.h>
 #include <sys/socket.h>
@@ -308,7 +327,7 @@ main(argc, argv)
 	    strcpy(channel, optarg);
 	    break;
 	  case 'd':		/* turn on debugging output */
-	    debug = 1;
+	    ++debug;
 	    break;
 	  case 'e':		/* expensive MX checking for *all* addresses */
 #ifdef	BIND
@@ -413,8 +432,11 @@ main(argc, argv)
 	  if (emptyline(file, sizeof file))
 	    break;
 
-	  /* if(logfp) fprintf(logfp,
-	     "%d#\t(fdcnt=%d, file:%s)\n",pid,countfds(),file); */
+	  if (debug > 1) { /* "DBGdiag:"-output */
+	    fprintf(stdout, "# (fdcnt=%d, file:%s)\n", countfds(), file);
+	    fflush(stdout);
+	  }
+		       
 
 #ifdef	BIND
 	  if (checkmx)
@@ -728,7 +750,9 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	 * Note that a good and useful approximation to "forever" is one day.
 	 * Murphy's Law you know: Connections will hang even when they can't.
 	 */
-	tout = timeout, timeout = 60*60*24;
+	/* RFC-1123 says: 10 minutes! */
+	tout = timeout;
+	timeout = ALARM_DOTTOOK;
 	r = smtpwrite(smtpfp, verboselog, 1, ".");
 	timeout = tout;
 	if (r != EX_OK)
@@ -784,10 +808,9 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 
 	writebuf(fp, (char *)NULL, 0);	/* magic init. */
 
-	alarm(1*60*60); /* Timeout in 1 hour per let_buffer full..
-			   BUFSIZ/h is damn slow.. It might not be that
-			   slow for some monster-files, but.. */
+	alarm(timeout);
 	gotalarm = 0;
+
 #ifndef	USE_MMAP
 	/* can we use cache of message body data */
 	if (convertmode == _CONVERT_NONE &&
@@ -968,15 +991,22 @@ writebuf(fp, buf, len)
 	register int n;
 	register int state;
 	static int save = 0;
+	static int alarmcnt = 0;
 
 	if (buf == NULL) {		/* magic initialization */
 	  save = 1;
+	  alarmcnt = ALARM_BLOCKSIZE;
 	  return 0;
 	}
 	state  = save;
-	for (cp = buf, n = len; n > 0; --n, ++cp) {
+	for (cp = buf, n = len; n > 0 && !gotalarm; --n, ++cp) {
 	  unsigned char c = *(unsigned char *)cp;
 
+	  if (--alarmcnt <= 0) {
+	    alarmcnt = ALARM_BLOCKSIZE;
+	    alarm(timeout);	/* re-arm it */
+	  }
+
 	  if (state && c != '\n') {
 	    state = 0;
 	    if (c == '.') {
@@ -1026,6 +1056,7 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 	register char *cp;
 	register int n;
 	static int column;
+	static int alarmcnt;
 	char *i2h = "0123456789ABCDEF";
 	int qp_chrs = 0;
 	int qp_val = 0;
@@ -1040,6 +1071,7 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 
 	if (buf == NULL) {
 	  column = -1; 
+	  alarmcnt = ALARM_BLOCKSIZE;
 	  return 0;
 	}
 
@@ -1054,6 +1086,12 @@ writemimeline(fp, buf, len, convertmode, lastcharp)
 	for (cp = buf, n = len; n > 0; --n, ++cp) {
 	  int c = *(unsigned char *)cp;
 	  ++column;
+
+	  if (--alarmcnt <= 0) {
+	    alarmcnt = ALARM_BLOCKSIZE;
+	    alarm(timeout);
+	  }
+
 	  if (convertmode == _CONVERT_8BIT) {
 	    if (c == '=' && qp_chrs == 0) {
 	      qp_val = 0;
@@ -1406,7 +1444,7 @@ smtpconn(host, fparr, verboselog)
 	  } else {
 	    retval = EX_UNAVAILABLE;
 	    for (i = 0; mxh[i].host != NULL; ++i) {
-	      if ((hp = gethostbyname(mxh[i].host)) == NULL) {
+	      if ((hp = gethostbyname((char*)mxh[i].host)) == NULL) {
 		free(mxh[i].host);
 		mxh[i].host = NULL;
 		firstmx = i+1;
@@ -2164,8 +2202,8 @@ getmxrr(host, mx, maxmx, depth, mxcntp)
 	    break;
 	  cp += n;
 
-	  hent = gethostbyname(buf); /* This resolves CNAME, should it
-					should not in case of MX server */
+	  hent = gethostbyname((char*)buf); /* This resolves CNAME, should it
+					       should not in case of MX server */
 	  if (!hent) {
 	    *mxcntp += 1;
 	    continue;		/* Well well.. spurious! */
@@ -2324,7 +2362,7 @@ getrrtype(host, hbsize, rrtype, cnamelevel)
 	    if (qdcount > 0 && strchr(host, '.') == NULL) {
 	      cp = (msgdata *)&answer + sizeof(HEADER);
 	      if (dn_expand((msgdata *)&answer,
-			    eom, cp, host, hbsize) >= 0) {
+			    eom, cp, (u_char*)host, hbsize) >= 0) {
 		if (host[0] == '\0') {
 		  host[0] = '.';
 		  host[1] = '\0';
@@ -2533,7 +2571,14 @@ notarystatsave(smtpline)
 char *smtpline;
 {
 	int len = strlen(smtpline)+8;
+#ifdef USE_ALLOCA
 	char *str = alloca(len);
+#else
+	char *str = malloc(len);	/* This leaks memory, but we
+					   propably can allow it for
+					   this ephemerally existing
+					   transporter.. */
+#endif
 	char *s = str;
 
 	*s++ = *smtpline++;
@@ -2572,19 +2617,3 @@ void getdaemon()
 	if (!pw) daemon_uid = 0; /* Let it be root, if nothing else */
 	else     daemon_uid = pw->pw_uid;
 }
-
-#if 0
-/* Debugging stuff.. */
-int countfds()
-{
-  int fds = getdtablesize();
-  int i;
-  int cnt = 0;
-
-  for (i=0; i<fds; ++i)
-    if (fcntl(i,F_GETFL) >= 0)
-      ++cnt;
-
-  return cnt;
-}
-#endif
